@@ -1,5 +1,3 @@
-#include <utility>
-
 #include "input_handler.hh"
 
 #include "buffer_manager.hh"
@@ -17,6 +15,10 @@
 #include "user_interface.hh"
 #include "utf8.hh"
 #include "window.hh"
+#include "word_db.hh"
+
+#include <utility>
+#include <limits>
 
 namespace Kakoune
 {
@@ -177,6 +179,7 @@ public:
           m_idle_timer{TimePoint::max(),
                        context().flags() & Context::Flags::Draft ?
                            Timer::Callback{} : [this](Timer&) {
+              RefPtr<InputMode> keep_alive{this}; // hook could trigger pop_mode()
               context().hooks().run_hook(Hook::NormalIdle, "", context());
           }},
           m_fs_check_timer{TimePoint::max(),
@@ -212,8 +215,8 @@ public:
 
     void on_disabled(bool temporary) override
     {
-        m_idle_timer.set_next_date(TimePoint::max());
-        m_fs_check_timer.set_next_date(TimePoint::max());
+        m_idle_timer.disable();
+        m_fs_check_timer.disable();
 
         if (not temporary and m_hooks_disabled)
         {
@@ -572,12 +575,12 @@ public:
         const Face cursor_face = m_faces["StatusCursor"];
 
         if (m_cursor_pos == str.char_length())
-            return DisplayLine{{ { fix_atom_text(str.substr(m_display_pos, width-1)), line_face },
+            return DisplayLine{{ { str.substr(m_display_pos, width-1).str(), line_face },
                                  { " "_str, cursor_face} } };
         else
-            return DisplayLine({ { fix_atom_text(str.substr(m_display_pos, m_cursor_pos - m_display_pos)), line_face },
-                                 { fix_atom_text(str.substr(m_cursor_pos,1)), cursor_face },
-                                 { fix_atom_text(str.substr(m_cursor_pos+1, width - m_cursor_pos + m_display_pos - 1)), line_face } });
+            return DisplayLine({ { str.substr(m_display_pos, m_cursor_pos - m_display_pos).str(), line_face },
+                                 { str.substr(m_cursor_pos,1).str(), cursor_face },
+                                 { str.substr(m_cursor_pos+1, width - m_cursor_pos + m_display_pos - 1).str(), line_face } });
     }
 private:
     CharCount  m_cursor_pos = 0;
@@ -753,6 +756,7 @@ public:
           m_auto_complete{context().options()["autocomplete"].get<AutoComplete>() & AutoComplete::Prompt},
           m_idle_timer{TimePoint::max(), context().flags() & Context::Flags::Draft ?
                            Timer::Callback{} : [this](Timer&) {
+                           RefPtr<InputMode> keep_alive{this}; // hook or m_callback could trigger pop_mode()
                            if (m_auto_complete and m_refresh_completion_pending)
                                refresh_completions(CompletionFlags::Fast);
                            if (m_line_changed)
@@ -770,11 +774,19 @@ public:
     {
         const String& line = m_line_editor.line();
 
+        auto can_auto_insert_completion = [&] {
+            const bool has_completions = not m_completions.candidates.empty();
+            const bool completion_selected = m_current_completion != -1;
+            const bool text_entered = m_completions.start != line.byte_count_to(m_line_editor.cursor_pos());
+            return (m_completions.flags & Completions::Flags::Menu) and
+                has_completions and
+                not completion_selected and
+                (not (m_completions.flags & Completions::Flags::NoEmpty) or text_entered);
+        };
+
         if (key == Key::Return)
         {
-            if ((m_completions.flags & Completions::Flags::Menu) and
-                m_current_completion == -1 and
-                not m_completions.candidates.empty())
+            if ((m_completions.flags & Completions::Flags::Menu) and can_auto_insert_completion())
             {
                 const String& completion = m_completions.candidates.front();
                 m_line_editor.insert_from(line.char_count_to(m_completions.start),
@@ -795,7 +807,8 @@ public:
             m_callback(line, PromptEvent::Validate, context());
             return;
         }
-        else if (key == Key::Escape or key == ctrl('c'))
+        else if (key == Key::Escape or key == ctrl('c') or
+                 ((key == Key::Backspace or key == ctrl('h')) and line.empty()))
         {
             history_push(line);
             context().print_status(DisplayLine{});
@@ -929,9 +942,40 @@ public:
                 m_refresh_completion_pending = true;
             }
         }
+        else if (key == ctrl('x'))
+        {
+            on_next_key_with_autoinfo(context(), "explicit-completion", KeymapMode::None,
+                [this](Key key, Context&) {
+                    m_explicit_completer = PromptCompleter{};
+
+                    if (key.key == 'f')
+                        use_explicit_completer([](const Context& context, StringView token) {
+                            return complete_filename(token, context.options()["ignored_files"].get<Regex>(),
+                                                     token.length(), FilenameFlags::Expand);
+                        });
+                    else if (key.key == 'w')
+                        use_explicit_completer([](const Context& context, StringView token) {
+                            CandidateList candidates;
+                            for_n_best(get_word_db(context.buffer()).find_matching(token),
+                                       100, [](auto& lhs, auto& rhs){ return rhs < lhs; },
+                                       [&](RankedMatch& m) {
+                                candidates.push_back(m.candidate().str());
+                                return true;
+                            });
+                            return candidates;
+                        });
+
+                    if (m_explicit_completer)
+                        refresh_completions(CompletionFlags::None);
+                }, "enter completion type",
+                "f: filename\n"
+                "w: buffer word\n");
+        }
         else if (key == ctrl('o'))
         {
+            m_explicit_completer = PromptCompleter{};
             m_auto_complete = not m_auto_complete;
+
             if (m_auto_complete)
                 refresh_completions(CompletionFlags::Fast);
             else if (context().has_client())
@@ -944,7 +988,7 @@ public:
         {
             try
             {
-                m_line_editor.reset(expand(m_line_editor.line(), context()), m_empty_text);
+                m_line_editor.reset(expand(line, context()), m_empty_text);
             }
             catch (std::runtime_error& error)
             {
@@ -959,6 +1003,12 @@ public:
         }
         else
         {
+            if (key == ' ' and
+                not (m_completions.flags & Completions::Flags::Quoted) and // if token is quoted, this space does not end it
+                can_auto_insert_completion())
+                m_line_editor.insert_from(line.char_count_to(m_completions.start),
+                                          m_completions.candidates.front());
+
             m_line_editor.handle_key(key);
             clear_completions();
             m_refresh_completion_pending = true;
@@ -995,17 +1045,37 @@ public:
     }
 
 private:
+    template<typename Completer>
+    void use_explicit_completer(Completer&& completer)
+    {
+        m_explicit_completer = [completer](const Context& context, CompletionFlags flags, StringView content, ByteCount cursor_pos) {
+            Optional<Token> last_token;
+            CommandParser parser{content.substr(0_byte, cursor_pos)};
+            while (auto token = parser.read_token(false))
+                last_token = std::move(token);
+
+            if (last_token and (last_token->pos + last_token->content.length() < cursor_pos))
+                last_token.reset();
+
+            auto token_start = last_token.map([&](auto&& t) { return t.pos; }).value_or(cursor_pos);
+            auto token_content = last_token.map([](auto&& t) -> StringView { return t.content; }).value_or(StringView{});
+
+            return Completions{token_start, cursor_pos, completer(context, token_content)};
+        };
+    }
+
     void refresh_completions(CompletionFlags flags)
     {
         try
         {
             m_refresh_completion_pending = false;
-            if (not m_completer)
+            auto& completer = m_explicit_completer ? m_explicit_completer : m_completer;
+            if (not completer)
                 return;
             m_current_completion = -1;
             const String& line = m_line_editor.line();
-            m_completions = m_completer(context(), flags, line,
-                                        line.byte_count_to(m_line_editor.cursor_pos()));
+            m_completions = completer(context(), flags, line,
+                                      line.byte_count_to(m_line_editor.cursor_pos()));
             const bool menu = (bool)(m_completions.flags & Completions::Flags::Menu);
             if (context().has_client())
             {
@@ -1068,13 +1138,14 @@ private:
         if (not temporary)
             context().print_status({});
 
-        m_idle_timer.set_next_date(TimePoint::max());
+        m_idle_timer.disable();
         if (context().has_client())
             context().client().menu_hide();
     }
 
     PromptCallback  m_callback;
     PromptCompleter m_completer;
+    PromptCompleter m_explicit_completer;
     const String   m_prompt;
     Face           m_prompt_face;
     Completions    m_completions;
@@ -1146,6 +1217,7 @@ public:
           m_auto_complete{context().options()["autocomplete"].get<AutoComplete>() & AutoComplete::Insert},
           m_idle_timer{TimePoint::max(), context().flags() & Context::Flags::Draft ?
                        Timer::Callback{} : [this](Timer&) {
+                           RefPtr<InputMode> keep_alive{this}; // hook could trigger pop_mode()
                            m_completer.update(m_auto_complete);
                            context().hooks().run_hook(Hook::InsertIdle, "", context());
                        }},
@@ -1169,7 +1241,7 @@ public:
 
     void on_disabled(bool temporary) override
     {
-        m_idle_timer.set_next_date(TimePoint::max());
+        m_idle_timer.disable();
 
         if (not temporary)
         {
@@ -1484,8 +1556,6 @@ private:
              kak_assert(false); // invalid for interactive insert
              break;
         }
-        if (mode != InsertMode::Append and mode != InsertMode::Replace)
-            selections.sort_and_merge_overlapping();
         selections.check_invariant();
         buffer.check_invariant();
     }
@@ -1526,6 +1596,7 @@ void InputHandler::pop_mode(InputMode* mode)
     kak_assert(m_mode_stack.back().get() == mode);
     kak_assert(m_mode_stack.size() > 1);
 
+    RefPtr<InputMode> keep_alive{mode}; // Ensure prev_name stays valid
     StringView prev_name = mode->name();
 
     current_mode().on_disabled(false);
