@@ -46,6 +46,7 @@ String to_json(Attribute attributes)
         { Attribute::FinalFg, "final_fg" },
         { Attribute::FinalBg, "final_bg" },
         { Attribute::FinalAttr, "final_attr" },
+        { Attribute::Strikethrough, "strikethrough" },
     };
 
     return "[" + join(attrs |
@@ -133,7 +134,7 @@ void rpc_call(StringView method, Args&&... args)
 }
 
 JsonUI::JsonUI()
-    : m_stdin_watcher{0, FdEvents::Read,
+    : m_stdin_watcher{0, FdEvents::Read, EventMode::Urgent,
                       [this](FDWatcher&, FdEvents, EventMode mode) {
         parse_requests(mode);
       }}, m_dimensions{24, 80}
