@@ -13,7 +13,7 @@
 #include "highlighters.hh"
 #include "insert_completer.hh"
 #include "json_ui.hh"
-#include "ncurses_ui.hh"
+#include "terminal_ui.hh"
 #include "option_types.hh"
 #include "parameters_parser.hh"
 #include "ranges.hh"
@@ -44,6 +44,14 @@ struct {
     unsigned int version;
     StringView notes;
 } constexpr version_notes[] = { {
+        20210828,
+        "» {+u}write <filename>{} will refuse to overwrite without {+u}-force{}\n"
+        "» {+u}$kak_command_fifo{} support\n"
+        "» {+u}set-option -remove{} support\n"
+        "» prompts auto select {+i}menu{} completions on space\n"
+        "» explicit completion support ({+b}<c-x>...{}) in prompts\n"
+        "» {+u}write -atomic{} was replaced with {+u}write -method <method>{}\n"
+    }, {
         20200901,
         "» daemon mode does not fork anymore\n"
     }, {
@@ -531,17 +539,17 @@ void register_options()
                        "space separated list of <key>=<value> options that are "
                        "passed to and interpreted by the user interface\n"
                        "\n"
-                       "The ncurses ui supports the following options:\n"
+                       "The terminal ui supports the following options:\n"
                        "    <key>:                        <value>:\n"
-                       "    ncurses_assistant             clippy|cat|dilbert|none|off\n"
-                       "    ncurses_status_on_top         bool\n"
-                       "    ncurses_set_title             bool\n"
-                       "    ncurses_enable_mouse          bool\n"
-                       "    ncurses_change_colors         bool\n"
-                       "    ncurses_wheel_up_button       int\n"
-                       "    ncurses_wheel_down_button     int\n"
-                       "    ncurses_wheel_scroll_amount   int\n"
-                       "    ncurses_shift_function_key    int\n",
+                       "    terminal_assistant             clippy|cat|dilbert|none|off\n"
+                       "    terminal_status_on_top         bool\n"
+                       "    terminal_set_title             bool\n"
+                       "    terminal_enable_mouse          bool\n"
+                       "    terminal_change_colors         bool\n"
+                       "    terminal_wheel_up_button       int\n"
+                       "    terminal_wheel_down_button     int\n"
+                       "    terminal_wheel_scroll_amount   int\n"
+                       "    terminal_shift_function_key    int\n",
                        UserInterface::Options{});
     reg.declare_option("modelinefmt", "format string used to generate the modeline",
                        "%val{bufname} %val{cursor_line}:%val{cursor_char_column} {{context_info}} {{mode_info}} - %val{client}@[%val{session}]"_str);
@@ -565,14 +573,14 @@ static bool convert_to_client_pending = false;
 
 enum class UIType
 {
-    NCurses,
+    Terminal,
     Json,
     Dummy,
 };
 
 UIType parse_ui_type(StringView ui_name)
 {
-    if (ui_name == "ncurses") return UIType::NCurses;
+    if (ui_name == "terminal") return UIType::Terminal;
     if (ui_name == "json") return UIType::Json;
     if (ui_name == "dummy") return UIType::Dummy;
 
@@ -604,7 +612,7 @@ std::unique_ptr<UserInterface> make_ui(UIType ui_type)
 
     switch (ui_type)
     {
-        case UIType::NCurses: return std::make_unique<NCursesUI>();
+        case UIType::Terminal: return std::make_unique<TerminalUI>();
         case UIType::Json: return std::make_unique<JsonUI>();
         case UIType::Dummy: return std::make_unique<DummyUI>();
     }
@@ -627,17 +635,17 @@ pid_t fork_server_to_background()
 
 std::unique_ptr<UserInterface> create_local_ui(UIType ui_type)
 {
-    if (ui_type != UIType::NCurses)
+    if (ui_type != UIType::Terminal)
         return make_ui(ui_type);
 
-    struct LocalUI : NCursesUI
+    struct LocalUI : TerminalUI
     {
         LocalUI()
         {
             set_signal_handler(SIGTSTP, [](int) {
                 if (ClientManager::instance().count() == 1 and
                     *ClientManager::instance().begin() == local_client)
-                    NCursesUI::instance().suspend();
+                    TerminalUI::instance().suspend();
                 else
                     convert_to_client_pending = true;
            });
@@ -652,7 +660,7 @@ std::unique_ptr<UserInterface> create_local_ui(UIType ui_type)
 
             if (fork_server_to_background())
             {
-                this->NCursesUI::~NCursesUI();
+                this->TerminalUI::~TerminalUI();
                 exit(local_client_exit);
             }
         }
@@ -730,6 +738,7 @@ int run_server(StringView session, StringView server_init,
 {
     static bool terminate = false;
     set_signal_handler(SIGTERM, [](int) { terminate = true; });
+    set_signal_handler(SIGINT, [](int) { terminate = true; });
     if ((flags & ServerFlags::Daemon) and session.empty())
     {
         write_stderr("-d needs a session name to be specified with -s\n");
@@ -737,7 +746,8 @@ int run_server(StringView session, StringView server_init,
     }
 
     EventManager        event_manager;
-    Server              server{session.empty() ? to_string(getpid()) : session.str()};
+    Server              server{session.empty() ? to_string(getpid()) : session.str(),
+                               (bool)(flags & ServerFlags::Daemon)};
 
     StringRegistry      string_registry;
     GlobalScope         global_scope;
@@ -759,6 +769,7 @@ int run_server(StringView session, StringView server_init,
 
     write_to_debug_buffer("*** This is the debug buffer, where debug info will be written ***");
 
+#ifdef KAK_DEBUG
     const auto start_time = Clock::now();
     UnitTest::run_all_tests();
 
@@ -768,8 +779,7 @@ int run_server(StringView session, StringView server_init,
         write_to_debug_buffer(format("running the unit tests took {} ms",
                                      duration_cast<milliseconds>(Clock::now() - start_time).count()));
     }
-
-    GlobalScope::instance().options().get_local_option("readonly").set<bool>(flags & ServerFlags::ReadOnly);
+#endif
 
     bool startup_error = false;
     if (not (flags & ServerFlags::IgnoreKakrc)) try
@@ -812,7 +822,10 @@ int run_server(StringView session, StringView server_init,
             {
                 Buffer *buffer = open_or_create_file_buffer(file);
                 if (flags & ServerFlags::ReadOnly)
+                {
                     buffer->flags() |= Buffer::Flags::ReadOnly;
+                    buffer->options().get_local_option("readonly").set(true);
+                }
             }
             catch (runtime_error& error)
             {
@@ -829,7 +842,7 @@ int run_server(StringView session, StringView server_init,
 
     try
     {
-        if (not (flags & ServerFlags::Daemon))
+        if (not server.is_daemon())
         {
             local_client = client_manager.create_client(
                  create_local_ui(ui_type), getpid(), {}, get_env_vars(), client_init, std::move(init_coord),
@@ -837,7 +850,7 @@ int run_server(StringView session, StringView server_init,
 
             if (startup_error and local_client)
                 local_client->print_status({
-                    "error during startup, see *debug* buffer for details",
+                    "error during startup, see `:buffer *debug*` for details",
                     local_client->context().faces()["Error"]
                 });
 
@@ -846,8 +859,7 @@ int run_server(StringView session, StringView server_init,
         }
 
         while (not terminate and
-               (not client_manager.empty() or server.negotiating() or
-                (flags & ServerFlags::Daemon)))
+               (not client_manager.empty() or server.negotiating() or server.is_daemon()))
         {
             client_manager.redraw_clients();
 
@@ -856,7 +868,8 @@ int run_server(StringView session, StringView server_init,
             bool allow_blocking = not client_manager.has_pending_inputs();
             while (event_manager.handle_next_events(EventMode::Normal, nullptr, allow_blocking))
             {
-                client_manager.process_pending_inputs();
+                if (client_manager.process_pending_inputs())
+                    break;
                 allow_blocking = false;
             }
             client_manager.process_pending_inputs();
@@ -959,8 +972,8 @@ int run_filter(StringView keystr, ConstArrayView<StringView> files, bool quiet,
         }
         if (not isatty(0))
         {
-            Buffer& buffer = *buffer_manager.create_buffer(
-                "*stdin*", Buffer::Flags::NoHooks, read_fd(0), InvalidTime);
+            Buffer& buffer = *create_buffer_from_string(
+                "*stdin*", Buffer::Flags::NoHooks, read_fd(0));
             apply_to_buffer(buffer);
             write_buffer_to_fd(buffer, 1);
             buffer_manager.delete_buffer(buffer);
@@ -991,7 +1004,7 @@ int run_pipe(StringView session)
 
 void signal_handler(int signal)
 {
-    NCursesUI::abort();
+    TerminalUI::restore_terminal();
     const char* text = nullptr;
     switch (signal)
     {
@@ -1048,7 +1061,7 @@ int main(int argc, char* argv[])
                    { "f", { true,  "filter: for each file, select the entire buffer and execute the given keys" } },
                    { "i", { true, "backup the files on which a filter is applied using the given suffix" } },
                    { "q", { false, "in filter mode, be quiet about errors applying keys" } },
-                   { "ui", { true, "set the type of user interface to use (ncurses, dummy, or json)" } },
+                   { "ui", { true, "set the type of user interface to use (terminal, dummy, or json)" } },
                    { "l", { false, "list existing sessions" } },
                    { "clear", { false, "clear dead sessions" } },
                    { "debug", { true, "initial debug option value" } },
@@ -1120,7 +1133,7 @@ int main(int argc, char* argv[])
 
         auto client_init = parser.get_switch("e").value_or(StringView{});
         auto server_init = parser.get_switch("E").value_or(StringView{});
-        const UIType ui_type = parse_ui_type(parser.get_switch("ui").value_or("ncurses"));
+        const UIType ui_type = parse_ui_type(parser.get_switch("ui").value_or("terminal"));
 
         if (auto keys = parser.get_switch("f"))
         {
