@@ -452,22 +452,24 @@ void for_each_codepoint(Context& context, NormalParams)
     selections.insert(strings, InsertMode::Replace);
 }
 
-void command(const Context& context, EnvVarMap env_vars)
+void command(const Context& context, EnvVarMap env_vars, char reg = 0)
 {
     if (not CommandManager::has_instance())
         throw runtime_error{"commands are not supported"};
 
     CommandManager::instance().clear_last_complete_command();
 
+    String default_command = context.main_sel_register_value(reg ? reg : ':').str();
+
     context.input_handler().prompt(
-        ":", {}, context.main_sel_register_value(':').str(),
+        ":", {}, default_command,
         context.faces()["Prompt"], PromptFlags::DropHistoryEntriesWithBlankPrefix,
         ':',
         [](const Context& context, CompletionFlags flags,
            StringView cmd_line, ByteCount pos) {
                return CommandManager::instance().complete(context, flags, cmd_line, pos);
         },
-        [env_vars = std::move(env_vars)](StringView cmdline, PromptEvent event, Context& context) {
+        [env_vars = std::move(env_vars), default_command](StringView cmdline, PromptEvent event, Context& context) {
             if (context.has_client())
             {
                 context.client().info_hide();
@@ -491,12 +493,9 @@ void command(const Context& context, EnvVarMap env_vars)
             if (event == PromptEvent::Validate)
             {
                 if (cmdline.empty())
-                    cmdline = context.main_sel_register_value(':');
-                else if (not is_blank(cmdline[0]))
-                    RegisterManager::instance()[':'].set(context, cmdline.str());
+                    cmdline = default_command;
 
-                CommandManager::instance().execute(
-                    cmdline, context, { {}, env_vars });
+                CommandManager::instance().execute(cmdline, context, { {}, env_vars });
             }
         });
 }
@@ -507,7 +506,7 @@ void command(Context& context, NormalParams params)
         { "count", to_string(params.count) },
         { "register", String{&params.reg, 1} }
     };
-    command(context, std::move(env_vars));
+    command(context, std::move(env_vars), params.reg);
 }
 
 BufferCoord apply_diff(Buffer& buffer, BufferCoord pos, StringView before, StringView after)
@@ -545,22 +544,22 @@ BufferCoord apply_diff(Buffer& buffer, BufferCoord pos, StringView before, Strin
 }
 
 template<bool replace>
-void pipe(Context& context, NormalParams)
+void pipe(Context& context, NormalParams params)
 {
     const char* prompt = replace ? "pipe:" : "pipe-to:";
+    String default_command = context.main_sel_register_value(params.reg ? params.reg : '|').str();
+
     context.input_handler().prompt(
-        prompt, {}, context.main_sel_register_value("|").str(), context.faces()["Prompt"],
+        prompt, {}, default_command, context.faces()["Prompt"],
         PromptFlags::DropHistoryEntriesWithBlankPrefix, '|',
         shell_complete,
-        [](StringView cmdline, PromptEvent event, Context& context)
+        [default_command](StringView cmdline, PromptEvent event, Context& context)
         {
             if (event != PromptEvent::Validate)
                 return;
 
             if (cmdline.empty())
-                cmdline = context.main_sel_register_value("|");
-            else
-                RegisterManager::instance()['|'].set(context, cmdline.str());
+                cmdline = default_command;
 
             if (cmdline.empty())
                 return;
@@ -626,22 +625,22 @@ void pipe(Context& context, NormalParams)
 }
 
 template<InsertMode mode>
-void insert_output(Context& context, NormalParams)
+void insert_output(Context& context, NormalParams params)
 {
     const char* prompt = mode == InsertMode::Insert ? "insert-output:" : "append-output:";
+    String default_command = context.main_sel_register_value(params.reg ? params.reg : '|').str();
+
     context.input_handler().prompt(
-        prompt, {}, context.main_sel_register_value("|").str(), context.faces()["Prompt"],
+        prompt, {}, default_command, context.faces()["Prompt"],
         PromptFlags::DropHistoryEntriesWithBlankPrefix, '|',
         shell_complete,
-        [](StringView cmdline, PromptEvent event, Context& context)
+        [default_command](StringView cmdline, PromptEvent event, Context& context)
         {
             if (event != PromptEvent::Validate)
                 return;
 
             if (cmdline.empty())
-                cmdline = context.main_sel_register_value("|");
-            else
-                RegisterManager::instance()['|'].set(context, cmdline.str());
+                cmdline = default_command;
 
             if (cmdline.empty())
                 return;
@@ -749,12 +748,14 @@ void paste_all(Context& context, NormalParams params)
     Vector<Selection> result;
     for (auto& ins_pos : insert_pos)
     {
-        ByteCount pos = 0;
+        ByteCount pos_offset = 0;
+        BufferCoord pos = ins_pos;
         for (auto offset : offsets)
         {
-            result.emplace_back(buffer.advance(ins_pos, pos),
-                                buffer.advance(ins_pos, offset-1));
-            pos = offset;
+            BufferCoord end = buffer.advance(pos, offset - pos_offset - 1);
+            result.emplace_back(pos, end);
+            pos = buffer.next(end);
+            pos_offset = offset;
         }
     }
     if (not result.empty())
@@ -965,7 +966,7 @@ void use_selection_as_search_pattern(Context& context, NormalParams params)
     const char reg = to_lower(params.reg ? params.reg : '/');
 
     context.print_status({
-        format("register '{}' set to '{}'", reg, fix_atom_text(pattern)),
+        format("register '{}' set to '{}'", reg, pattern),
         context.faces()["Information"] });
 
     RegisterManager::instance()[reg].set(context, {pattern});
@@ -1133,14 +1134,22 @@ void keep(Context& context, NormalParams params)
     });
 }
 
-void keep_pipe(Context& context, NormalParams)
+void keep_pipe(Context& context, NormalParams params)
 {
+    String default_command = context.main_sel_register_value(params.reg ? params.reg : '|').str();
     context.input_handler().prompt(
-        "keep pipe:", {}, {}, context.faces()["Prompt"],
+        "keep pipe:", {}, default_command, context.faces()["Prompt"],
         PromptFlags::DropHistoryEntriesWithBlankPrefix, '|', shell_complete,
-        [](StringView cmdline, PromptEvent event, Context& context) {
+        [default_command](StringView cmdline, PromptEvent event, Context& context) {
             if (event != PromptEvent::Validate)
                 return;
+
+            if (cmdline.empty())
+                cmdline = default_command;
+
+            if (cmdline.empty())
+                return;
+
             const Buffer& buffer = context.buffer();
             auto& shell_manager = ShellManager::instance();
             Vector<Selection> keep;
@@ -1370,8 +1379,8 @@ void select_object(Context& context, NormalParams params)
     }, get_title(),
     build_autoinfo_for_mapping(context, KeymapMode::Object,
         {{{'b','(',')'}, "parenthesis block"},
-         {{'B','{','}'}, "braces block"},
-         {{'r','[',']'}, "brackets block"},
+         {{'B','{','}'}, "brace block"},
+         {{'r','[',']'}, "bracket block"},
          {{'a','<','>'}, "angle block"},
          {{'"','Q'},     "double quote string"},
          {{'\'','q'},    "single quote string"},
@@ -2097,7 +2106,7 @@ void remove_selection(Context& context, NormalParams p)
     if (index >= selections.size())
         throw runtime_error{format("invalid selection index: {}", index)};
     if (selections.size() == 1)
-        throw runtime_error{"cannot remove the last selection"};
+        throw no_selections_remaining{};
 
     selections.remove(index);
     selections.check_invariant();
@@ -2137,6 +2146,22 @@ void merge_consecutive(Context& context, NormalParams params)
     context.selections().merge_consecutive();
 }
 
+void merge_overlapping(Context& context, NormalParams params)
+{
+    ensure_forward(context, params);
+    context.selections().merge_overlapping();
+}
+
+void duplicate_selections(Context& context, NormalParams params)
+{
+    SelectionList& sels = context.selections();
+    Vector<Selection> new_sels;
+    const int count = params.count ? params.count : 2;
+    for (const auto& sel : sels)
+        new_sels.insert(new_sels.end(), count, sel);
+    context.selections().set(std::move(new_sels), sels.main_index() * count);
+}
+
 void force_redraw(Context& context, NormalParams)
 {
     if (context.has_client())
@@ -2249,6 +2274,8 @@ static constexpr HashMap<Key, NormalCmd, MemoryDomain::Undefined, KeymapBackend>
     { {alt(';')}, {"swap selections cursor and anchor", flip_selections} },
     { {alt(':')}, {"ensure selection cursor is after anchor", ensure_forward} },
     { {alt('_')}, {"merge consecutive selections", merge_consecutive} },
+    { {'+'}, {"duplicate each selection", duplicate_selections} },
+    { {alt('+')}, {"merge overlapping selections", merge_overlapping} },
 
     { {'w'}, {"select to next word start", repeated<&select<SelectMode::Replace, select_to_next_word<Word>>>} },
     { {'e'}, {"select to next word end", repeated<select<SelectMode::Replace, select_to_next_word_end<Word>>>} },
